@@ -31,18 +31,24 @@
 #include <inttypes.h>
 
 #include <glib.h>
-#include <gdbus/gdbus.h>
 
+#include "lib/bluetooth.h"
+#include "lib/sdp.h"
 #include "lib/uuid.h"
+
+#include "gdbus/gdbus.h"
+
 #include "src/plugin.h"
 #include "src/adapter.h"
 #include "src/device.h"
 #include "src/dbus-common.h"
 #include "src/profile.h"
 
-#include "glib-helper.h"
-#include "log.h"
-#include "error.h"
+#include "src/uuid-helper.h"
+#include "src/log.h"
+#include "src/error.h"
+#include "src/shared/queue.h"
+
 #include "avdtp.h"
 #include "media.h"
 #include "transport.h"
@@ -268,9 +274,6 @@ static void endpoint_reply(DBusPendingCall *call, void *user_data)
 
 		/* Clear endpoint configuration in case of NO_REPLY error */
 		if (dbus_error_has_name(&err, DBUS_ERROR_NO_REPLY)) {
-			if (request->cb)
-				request->cb(endpoint, NULL, size,
-							request->user_data);
 			clear_endpoint(endpoint);
 			dbus_message_unref(reply);
 			dbus_error_free(&err);
@@ -1031,6 +1034,8 @@ static const char *repeat_to_loop_status(const char *value)
 		return "Track";
 	else if (strcasecmp(value, "alltracks") == 0)
 		return "Playlist";
+	else if (strcasecmp(value, "group") == 0)
+		return "Playlist";
 
 	return NULL;
 }
@@ -1056,12 +1061,17 @@ static int set_setting(const char *key, const char *value, void *user_data)
 	const char *iface = MEDIA_PLAYER_INTERFACE;
 	DBusMessage *msg;
 	DBusMessageIter iter;
+	const char *curval;
 
 	DBG("%s = %s", key, value);
 
-	if (!g_hash_table_lookup(mp->settings, key))
+	curval = g_hash_table_lookup(mp->settings, key);
+	if (!curval)
 		return -EINVAL;
 
+	if (strcasecmp(curval, value) == 0)
+		return 0;
+
 	msg = dbus_message_new_method_call(mp->sender, mp->path,
 					DBUS_INTERFACE_PROPERTIES, "Set");
 	if (msg == NULL) {
@@ -1820,10 +1830,10 @@ static const GDBusMethodTable media_methods[] = {
 		NULL, register_endpoint) },
 	{ GDBUS_METHOD("UnregisterEndpoint",
 		GDBUS_ARGS({ "endpoint", "o" }), NULL, unregister_endpoint) },
-	{ GDBUS_EXPERIMENTAL_METHOD("RegisterPlayer",
+	{ GDBUS_METHOD("RegisterPlayer",
 		GDBUS_ARGS({ "player", "o" }, { "properties", "a{sv}" }),
 		NULL, register_player) },
-	{ GDBUS_EXPERIMENTAL_METHOD("UnregisterPlayer",
+	{ GDBUS_METHOD("UnregisterPlayer",
 		GDBUS_ARGS({ "player", "o" }), NULL, unregister_player) },
 	{ },
 };
